	SUBROUTINE	compandsave( 	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
					PLAN1X, PLAN1Y, lowxv, upxv, lowyv, upyv, &
					UINI, VINI, AUINI, MA0, MASSCHECK, PLOTARRAYU, &
					PLOTARRAYV, T1, T2, thefileu, thefilev, TIME, plotnum)  
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	compandsave saves the Fourier coefficients and the numerical solution
	!	in a binary file, by calling the subroutine writebinfile,
	!	and calculate the test for the mass conservation.
	!
	! .. SPECIFICATION ..
	!	compandsave( Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
	!			PLAN1X, PLAN1Y, lowxv, upxv, lowyv, upyv, &
	!			UINI, VINI, AUINI, MA0, MASSCHECK, PLOTARRAYU, &
	!			PLOTARRAYV, T1, T2, thefileu, thefilev, TIME, plotnum  ) 
	!
	!	INTEGER(kind=4)	lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), &
	!			Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, thefileu, thefilev
	!	INTEGER(kind=8)	PLAN2X, PLAN2Y, PLAN1X, PLAN1Y
	!	DOUBLE PRECISION	MA0, MASSCHECK, PLOTARRAYU(1:Nx,lowyv(t_p):upyv(t_p)), &
	!				PLOTARRAYV(1:NY,lowxv(t_p):upxv(t_p))
	!	COMPLEX(kind=8)	AUINI(1:Nx, lowyv(t_p):upyv(t_p)), UINI(1:Nx, lowyv(t_p):upyv(t_p))
	!			VINI(1:Ny, lowxv(t_p):upxv(t_p)), T1(1:ydiff,1:Nx), T2(1:ydiff,1:Nx)
	!
	! .. INPUT ..
	! Ny  		= The number of grid points in the vertical direction 
	! Nx		= The number of grid points in the horizontal direction
	! info_p	= MPI error return integer
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! lowyv		= lower array index for data to be saved to file
	! upyv		= upper array index for data to be saved to file
	! lowxv		= lower array index for data to be saved to file
	! upxv		= upper array index for data to be saved to file
	! MA0		= initial mass numerically calculated via FFT (in both spatial coordinates)
	! MASSCHECK	= test of the numerical precision provided by mass conservation
	! PLAN2X	= Backward fft in X direction
	! PLAN2Y	= Backward fft in Y direction
	! PLAN1X	= Forward fft in X direction
	! PLAN1Y	= Forward fft in Y direction
	! Kx		= X fourier frequencies
	! Kxb		= X fourier frequencies, modified so do not get division by zero
	! Ky		= Y fourier frequencies
	! TIME		= Time 
	! UINI		= u, solution of DS, size: [Nx Ny/n_p]
	! VINI 		= FFT(u), (FFT in both spatial coordinates), size: [Ny Nx/n_p]
	! AUINI		= Absolute Value of u, squared; size: [Nx Ny/n_p]
	! PLOTARRAYU	= Array used to save real data for plotting; size: [Nx Ny/n_p]
	! PLOTARRAYV	= Array used to save real data for plotting; size: [Ny Nx/n_p]
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! writebinfile	-- saves a two dimensional field in native binary, double format using MPIIO
	! fft2dFor  	-- Computes the 2d-FFT forward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	! fft2dBack	-- Computes the 2d-FFT backward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	!---------------------------------------------------------------------------------------------

	USE mpi
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					::  info_p, thefileu, thefilev, plotnum
	INTEGER(KIND=8), INTENT(INOUT)					::  PLAN1X, PLAN1Y
	DOUBLE PRECISION, INTENT(IN)					::  time, MA0  
	DOUBLE PRECISION, INTENT(OUT)					::  MASSCHECK 
	! .. Array Arguments ..
	INTEGER(KIND=4), DIMENSION(0:n_p-1), INTENT(IN)							::  lowxv, upxv, lowyv, upyv
	COMPLEX(KIND=8), DIMENSION(1:Nx,lowyv(t_p):upyv(t_p)), INTENT(INOUT) 	::  UINI, AUINI
	COMPLEX(KIND=8), DIMENSION(1:Ny,lowxv(t_p):upxv(t_p)), INTENT(INOUT) 	::  VINI
	DOUBLE PRECISION, DIMENSION(1:Nx,lowyv(t_p):upyv(t_p)), INTENT(INOUT) 	::  PLOTARRAYU
	DOUBLE PRECISION, DIMENSION(1:Ny,lowxv(t_p):upxv(t_p)), INTENT(INOUT) 	::  PLOTARRAYV
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT)						::  T1, T2
	! .. Local scalars ..
	DOUBLE PRECISION							::  MAF
	INTEGER(KIND=4)								::  j, n, ind, stat
	! .. Local arrays ..
	CHARACTER*100								::  nomfile, DataDirName
	CHARACTER*10								::  number_file
	! .. Executable statements ..	
	CONTINUE

	DO j=lowyv(t_p),upyv(t_p)
		PLOTARRAYU(1:NX,j) = ABS(UINI(1:Nx,j))**2
	END DO
!	Save data
!	"Recuperation" of the name of the directory where data have to be saved
	CALL GET_ENVIRONMENT_VARIABLE(NAME="ddir", VALUE=DataDirName, STATUS=stat)
	IF (stat.NE.0) STOP
	nomfile = trim(DataDirName)//'/u'
	ind = index(nomfile,' ') - 1
	WRITE(number_file,'(i0)') 10000000+plotnum
	nomfile = nomfile(1:ind)//number_file
	ind = index(nomfile,' ') - 1
	nomfile = nomfile(1:ind)//'.datbin'
	CALL WRITEBINFILE(NX,NY,lowyv(t_p),upyv(t_p),n_p,t_p,nomfile,thefileu,info_p,PLOTARRAYU) 
!	RECUP COEF FOURIER
	CALL fft2dFor(  Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
			PLAN1X, PLAN1Y, lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), &
			upyv(0:n_p-1), UINI(1:Nx,lowyv(t_p):upyv(t_p)), &
			VINI(1:Ny,lowxv(t_p):upxv(t_p)), T1(1:ydiff*Nx),T2(1:ydiff*Nx))  
!	Save datas
	nomfile = trim(DataDirName)//'/v'
	ind = index(nomfile,' ') - 1
	WRITE(number_file,'(i0)') 10000000+plotnum
	nomfile = nomfile(1:ind)//number_file
	ind = index(nomfile,' ') - 1
	nomfile = nomfile(1:ind)//'.datbin'	
	DO n = 1,n_p
		PLOTARRAYV(lowyv(n-1):upyv(n-1), lowxv(t_p):upxv(t_p))=&
		LOG10(ABS(VINI(lowyv(n-1):upyv(n-1), lowxv(t_p):upxv(t_p)))) 
	END DO
	CALL WRITEBINFILE(NY,NX,lowxv(t_p),upxv(t_p),n_p,t_p,nomfile,thefilev,info_p,PLOTARRAYV) 
	CALL fft2dBack(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
			PLAN1X, PLAN1Y, lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), &
			upyv(0:n_p-1), VINI(1:Ny,lowxv(t_p):upxv(t_p)), &
			UINI(1:Nx,lowyv(t_p):upyv(t_p)), T1(1:ydiff*Nx),T2(1:ydiff*Nx) )  
	! .. test conservation mass ..
	CALL fft2dFor(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, PLAN1X, PLAN1Y, &
			lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), &
			AUINI(1:Nx,lowyv(t_p):upyv(t_p)), VINI(1:Ny,lowxv(t_p):upxv(t_p)), T1(1:ydiff*Nx),T2(1:ydiff*Nx) )  
	IF (t_p.EQ.0) THEN
		MAF = ABS(VINI(1,1))
		MASSCHECK= ABS(1D0 -MAF/MA0)

		! .. Save Time and Mass ..
		nomfile = trim(DataDirName)//'/Time.dat'
		IF (plotnum.eq.0) THEN
			! .. If this is the first plot, start at the beginning of the file ..
			OPEN(unit=1,FILE=nomfile,status="UNKNOWN") 
			REWIND(1)
			WRITE(1,'(e15.3)') time
			CLOSE(1)
		ELSE	
			OPEN(unit=1,FILE=nomfile,status="OLD",position="APPEND") 
			WRITE(1,'(e15.3)') time
			CLOSE(1)
		END IF		
		
		nomfile = trim(DataDirName)//'/Masscheck.dat'	
		IF (plotnum.eq.0) THEN
			! .. If this is the first plot, start at the beginning of the file ..
			OPEN(unit=1,FILE=nomfile,status="UNKNOWN") 
			REWIND(1)
			WRITE(1,'(e15.3)') MASSCHECK
			CLOSE(1)
		ELSE	
			OPEN(unit=1,FILE=nomfile,status="OLD",position="APPEND") 
			WRITE(1,'(e15.3)') MASSCHECK
			CLOSE(1)
		END IF		

		nomfile = trim(DataDirName)//'/Mass.dat'	
		IF (plotnum.eq.0) THEN
			! .. If this is the first plot, start at the beginning of the file ..
			OPEN(unit=1,FILE=nomfile,status="UNKNOWN") 
			REWIND(1)
			WRITE(1,'(e15.3)') MAF
			CLOSE(1)
		ELSE	
			OPEN(unit=1,FILE=nomfile,status="OLD",position="APPEND") 
			WRITE(1,'(e15.3)') MAF
			CLOSE(1)
		END IF		
	ENDIF
	plotnum=plotnum+1	

	END SUBROUTINE compandsave

