	SUBROUTINE fftinit(	fftx,ffty, Nx, Ny, PLAN1X, PLAN1Y, PLAN2X, PLAN2Y )
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	fftinit initializes PLAN1X, PLAN2X, PLAN1Y, PLAN2Y,
	!	data structures containing all the information that 
	!	fftw needs in order to compute the 1D Fourier transform, 
	!	forward and backward, in x and y directions, respectively
	!
	! .. SPECIFICATION ..
	!
	!	fftinit(	fftx, ffty, Nx, Ny, PLAN1X, PLAN1Y, PLAN2X, PLAN2Y )
	!
	!	COMPLEX(kind=8)	fftx(1:Nx), ffty(1:Ny)
	!	INTEGER(kind=4)	Nx, Ny
	!	INTEGER(kind=8)	PLAN1X, PLAN1Y, PLAN2X, PLAN2Y
	!	
	! .. INPUT ..
	! fftx		= space for FFTW to test plans in X direction
	! ffty		= space for FFTW to test plans in Y direction
	! Nx		= The number of grid points in the horizontal direction
	! Ny		= The number of grid points in the vertical direction 
	! PLAN1X	= Forward fft in X direction
	! PLAN1Y	= Forward fft in Y direction
	! PLAN2X	= Backward fft in X direction
	! PLAN2Y	= Backward fft in Y direction
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! .. EXTERNAL ROUTINES REQUIRED ..
	!	MPI library
	!	FFTW3
	!---------------------------------------------------------------------------------------------


	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN) 		::  Nx, Ny 
	INTEGER(KIND=8), INTENT(OUT) 		::  PLAN1X, PLAN1Y, PLAN2X, PLAN2Y
	! .. Array arguments ..
	COMPLEX, DIMENSION(1:Nx), INTENT(OUT) 	::  fftx
	COMPLEX, DIMENSION(1:Ny), INTENT(OUT)	::  ffty
	! .. Local Scalars ..
	INTEGER(KIND=4), PARAMETER 		::  FFTW_BACKWARD=1, FFTW_FORWARD=-1, FFTW_ESTIMATE=64, FFTW_EXHAUSTIVE=8
	! .. Executable statements ..	
	CONTINUE

  	! set up ffts
   	CALL dfftw_plan_dft_1d(PLAN1X,Nx,fftx(1:Nx),fftx(1:Nx),FFTW_FORWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN2X,Nx,fftx(1:Nx),fftx(1:Nx),FFTW_BACKWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN1Y,Ny,ffty(1:Ny),ffty(1:Ny),FFTW_FORWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN2Y,Ny,ffty(1:Ny),ffty(1:Ny),FFTW_BACKWARD,FFTW_ESTIMATE)

	END SUBROUTINE fftinit
