




		SUBROUTINE WRITEBINFILE(Nx,Ny,lowyv,upyv,n_p,t_p,nomfile,thefile,info_p,LAVINI) 
		!-----------------------------------------------------------------------------------------------------------------------------
		! PURPOSE
		!
		! WRITEBINFILE start saving a two dimensional field in binary, little endian, double format using asynchronous MPIIO 
		!
		! SPECIFICATION
		!
		! WRITEBINFILE(LAVINI(1:NX,lowyv:upyv),info_p,n_p, t_p,lowyv(t_p),upyv(t_p),NX,NY,nomfile,thefile)
		!
		! INTEGER			info_p, n_p, t_p,lowyv,upyv,NX,NY
		! DOUBLE PRECISION	LAVINI(1:NX,lowyv:upyv)
		! CHARACTER*100		nomfile
		!
		! INPUT ..
		! NY  		= The number of grid points in the vertical direction 
		! NX		= The number of grid points in the horizontal direction
		! info_p	= MPI error return integer
		! n_p		= Number of MPI processes
		! t_p		= Number of this MPI process
		! lowyv		= lower array index for data to be saved to file
		! upyv		= upper array index for data to be saved to file
		! nomfile	= name of file to be saved
		! LAVINI 	= Data to be saved to disk
		! thefile	= handle for file
		! REFERENCES
		!
		! ACCURACY
		!		
		! ERROR INDICATORS AND WARNINGS
		!
		! FURTHER COMMENTS
		!-----------------------------------------------------------------------------------------------------------------------------------------
		! External routines required
		! MPI library
		!
		USE MPI
		! .. Implicit none statement ..		
		IMPLICIT NONE
		! .. Scalar arguments ..
		INTEGER(KIND=4), INTENT(IN)		 			:: n_p, t_p, lowyv, upyv, Nx, Ny 
		INTEGER(KIND=4), INTENT(INOUT)		 		:: info_p, thefile		
		! .. Array arguments ..
		DOUBLE PRECISION, INTENT(IN), DIMENSION(1:Nx,lowyv:upyv)  :: LAVINI 
		CHARACTER*100,   INTENT(IN)  					:: nomfile
		! .. Local Scalars ..
      		INTEGER(KIND=MPI_OFFSET_KIND)				:: size	
		INTEGER(KIND=4)									:: count 
		! .. Executable statements ..	
		CONTINUE		
		! .. Open file ..
		CALL MPI_FILE_OPEN(MPI_COMM_WORLD, nomfile, &
             		MPI_MODE_WRONLY + MPI_MODE_CREATE, MPI_INFO_NULL, thefile, info_p)
		size=0
		CALL MPI_FILE_SET_SIZE(thefile,size,info_p)
		count=Nx*Ny/n_p
   		CALL MPI_FILE_WRITE_ORDERED_BEGIN(thefile,LAVINI,count,MPI_DOUBLE_PRECISION,info_p)
		END SUBROUTINE WRITEBINFILE
