	SUBROUTINE init(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
				PLANX, PLANY, Lx, Ly, pi, RHO, &
				MA0, lowxv, upxv, lowyv, upyv,  &
				Ky_d , Kx_d, &
				UINI_d, AUINI_d, VINI_d, T1_d, T2_d, T1, T2 )
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	init initializes the grid points of computation, fourier frequencies,
	!	the initial data, uini, and the initial mass are computed
	!
	! .. SPECIFICATION ..
	!	init( Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
	!		PLANX, PLANY, Lx, Ly, pi, RHO, &
	!		MA0, lowxv, upxv, lowyv, upyv, &
	!		lowxv_d, upxv_d, lowyv_d, upyv_d, Kx_d, Ky_d, &
	!		UINI_d, AUINI_d, VINI_d, T1, T2 )  
	!
	!	INTEGER(kind=4)	lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), & 
	!			lowxv_d(0:n_p-1), upxv_d(0:n_p-1), lowyv_d(0:n_p-1), upyv_d(0:n_p-1), &
	!			Nx, Ny, n_p, t_p, xdiff, ydiff, info_p
	!	INTEGER(kind=8)	PLANX, PLANY
	!	REAL(KIND=8)	Kx_d(lowxv(t_p):upxv(t_p)), Ky_d(1:Ny), Lx, Ly, pi, RHO, MA0
	!	COMPLEX(kind=8)	AUINI_d(1:Nx, lowyv(t_p):upyv(t_p)), UINI_d(1:Nx, lowyv(t_p):upyv(t_p)), &
	!			VINI_d(1:Ny, lowxv(t_p):upxv(t_p)),T1(1:ydiff,1:Nx), T2(1:ydiff,1:Nx)
	!
	! .. INPUT ..
	! Ny		= The number of grid points in the vertical direction 
	! Nx		= The number of grid points in the horizontal direction
	! info_p	= MPI error return integer
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! lowyv		= lower array index for data to be saved to file
	! upyv		= upper array index for data to be saved to file
	! lowxv		= lower array index for data to be saved to file
	! upxv		= upper array index for data to be saved to file
	! lowyv_d	= lower array index for data to be saved to file
	! upyv_d	= upper array index for data to be saved to file
	! lowxv_d	= lower array index for data to be saved to file
	! upxv_d	= upper array index for data to be saved to file
	! RHO		= +1 for the defocusing case and -1 for the focusing case
	! MA0		= initial mass numerically calculated via FFT
	! MA0C		= initial mass numerically calculated via FFT
	! PLANX		= FFT plan in X direction
	! PLANY		= FFT plan in Y direction
	! Kx_d		= X fourier frequencies
	! Ky_d		= Y fourier frequencies
	! Kx		= X fourier frequencies
	! kxt		= X fourier frequencies, to be written
	! Ky		= Y fourier frequencies
	! Ly		= Length in y direction
	! Lx		= Length in x direction
	! pi		= the number pi
	! UINI_d	= u, solution of DS, size: [Nx Ny/n_p]
	! VINI_d 	= FFT(u), (FFT in both spatial coordinates), size: [Ny Nx/n_p]
	! AUINI_d	= Absolute Value of u, squared; size: [Nx Ny/n_p]
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	! T1		= swap space for FFT
	! T2		= swap space for FFT
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!-----------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! fft2dFor:	-- Computes the 2d-FFT forward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once,
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	!-----------------------------------------------------------------------------------------------
	USE mpi
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, info_p, xdiff, ydiff 
	INTEGER(KIND=4), INTENT(INOUT)					::  PLANX, PLANY
	REAL(KIND=8), INTENT(IN)						::  Lx, Ly, pi, RHO
	REAL(KIND=8), INTENT(OUT)						::  MA0
	! .. Array Arguments ..
	INTEGER(KIND=4), DIMENSION(0:n_p-1), INTENT(IN)			::  lowxv, upxv, lowyv, upyv
	REAL(KIND=8), DEVICE, DIMENSION(t_p*xdiff+1:(t_p+1)*xdiff), INTENT(OUT)		::  kx_d
	REAL(KIND=8), DEVICE, DIMENSION(1:Ny), INTENT(OUT)						::  ky_d  
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:Nx,lowyv(t_p):upyv(t_p)), INTENT(OUT)	::  UINI_d, AUINI_d  
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:Ny,lowxv(t_p):upxv(t_p)), INTENT(OUT)	::  VINI_d  
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT)		::  T1_d, T2_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT)				::  T1, T2
	! .. Local Scalars ..
	INTEGER(KIND=4)													::  i, j, n, stat
	REAL(KIND=8)													::  kxt,x,y
	COMPLEX(KIND=8)													::  MA0C
	! .. Local Arrays ..
	REAL(KIND=8), DIMENSION(lowxv(t_p):upxv(t_p))					::  kx
	REAL(KIND=8), DIMENSION(1:Ny)									::  ky  
	CHARACTER*100													::  name_file, DataDirName
	! .. Executable statements ..
	CONTINUE

	CALL GET_ENVIRONMENT_VARIABLE(NAME="ddir",VALUE=DataDirName,STATUS=stat)
	IF (stat.ne.0) THEN
		IF(t_p.eq.0) THEN
			PRINT*,"Need to set environment variable ddir to the name of the directory for output"
		END IF
		STOP
	END IF
!	Fourier frequencies
	DO i = 1, Nx/2
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx(i) = REAL(i-1,KIND=8)/Lx 
		ENDIF
	END DO
	IF ((Nx/2 + 1 .GE.lowxv(t_p)).AND.(Nx/2 + 1 .LE.upxv(t_p))) THEN
		Kx( Nx/2 + 1 ) = 0.0d0 
	ENDIF
	DO i = Nx/2+2, Nx  
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx( i) = -1.0d0*REAL(1-i+Nx,KIND=8)/Lx  
		ENDIF
	END DO      
	DO j = 1, Ny/2
		Ky(j) = REAL(j-1,KIND(0d0))/Ly 
	END DO
	Ky( Ny/2 + 1 ) = 0.0d0 
	DO j = 1, Ny/2-1
		Ky( Ny/2 + 1 + j ) = -Ky(1-j+Ny/2) 
	END DO    
	! Transfer frequency information to GPU
	Kx_d=Kx
	Ky_d=Ky
	IF(t_p.eq.0) THEN
		! .. Save x locations .. 
		name_file = trim(DataDirName)//'/X.dat'
		OPEN(unit=11,FILE=name_file,status="UNKNOWN") 
		REWIND(11)
		DO i=1,NX
			WRITE(11,*) Lx*pi*( -1D0 + 2D0*REAL(i-1,KIND(0d0))/REAL(Nx,KIND=8) )
		END DO
		CLOSE(11)
		
		! .. Save y locations ..
		name_file = trim(DataDirName)//'/Y.dat'
		OPEN(unit=12,FILE=name_file,status="UNKNOWN") 
		REWIND(12)
		DO j=1,NY
			WRITE(12,*) Ly*pi*( -1D0 + 2D0*REAL(j-1,KIND(0d0))/REAL(Ny,KIND=8) )
		END DO
		CLOSE(12)

		! .. Save y frequencies ..
		name_file = trim(DataDirName)//'/KY.dat'
		OPEN(unit=13,FILE=name_file,status="UNKNOWN") 
		REWIND(13)
		DO j=1,NY
			WRITE(13,*) KY(j)
		END DO
		CLOSE(13)	

		! .. Save x frequencies .. 
		name_file = trim(DataDirName)//'/KX.dat'
		OPEN(unit=14,FILE=name_file,status="UNKNOWN") 
		REWIND(14)
		DO i=1,NX
			IF((i.ge.1).and.(i.le.Nx/2)) THEN
				kxt = REAL(i-1,KIND=8)/Lx 
			ELSEIF(i.eq.1+Nx/2) THEN
				Kxt = 0.0d0
			ELSEIF((i.ge.Nx/2+1).and.(i.le.Nx)) THEN 
				Kxt = -1.0d0*REAL(1-i+Nx,KIND=8)/Lx 
			END IF    			
			WRITE(14,*) Kxt
		END DO
		CLOSE(14)
	END IF
		
	!Initial data u=e(-(X2+Y2)), UINI, AND ITS ABSOLUTE SQUARED 
	!$cuf kernel do <<< *, * >>>   
	DO j = t_p*ydiff+1,(t_p+1)*ydiff
		y = Ly*pi*( -1D0 + 2D0*REAL(j-1,KIND(0d0))/REAL(Ny,KIND=8) )
		DO i=1,Nx
			x = Lx*pi*( -1D0 + 2D0*REAL(i-1,KIND(0d0))/REAL(Nx,KIND=8) )
			UINI_d(i,j) = 10D0*EXP( -0.10D0*(x**2+y**2) )*SIN(x**2+y**2+1D0)
		END DO
	END DO   
	
  	!$cuf kernel do <<< *, * >>> 
	DO j = t_p*ydiff+1,(t_p+1)*ydiff
		DO i=1,Nx
			AUINI_d(i,j) = ABS(UINI_d(i,j))**2
		END DO		
	END DO    
	 
	CALL fft2dFor(Nx, Ny , n_p, t_p, xdiff, ydiff, info_p, &
			PLANX, PLANY, &
			lowxv, upxv, lowyv, upyv, &
			AUINI_d, VINI_d, T1_d, T2_d,T1, T2 ) 

	IF (t_p.EQ.0) THEN
		MA0C=VINI_d(1,1)
		MA0 = ABS(MA0C)
		PRINT *,'initial mass',MA0
	ENDIF

	END SUBROUTINE init
