	SUBROUTINE init(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
				PLAN1X, PLAN1Y, PLAN2X, PLAN2Y, Lx, Ly, pi, &
				lowxv, upxv, lowyv, upyv,  &
				Ky , Kx, Ky2 , Kx2, omeg, omeghat, T1, T2, ffttemp, ffttemphat)
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	init initializes the grid points of computation, fourier frequencies,
	!	the initial data, uini, and the initial mass are computed
	!
	! .. SPECIFICATION ..
	!	init( Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
	!		PLANX, PLANY, Lx, Ly, pi,  &
	!		lowxv, upxv, lowyv, upyv, &
	!		Kx, Ky, Kx2, Ky2, &
	!		omeg, omeghat, T1, T2, fftemp1, fftemp2 )  
	!
	!	INTEGER(kind=4)	lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), & 
	!			lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), &
	!			Nx, Ny, n_p, t_p, xdiff, ydiff, info_p
	!	INTEGER(kind=8)	PLAN1X, PLAN1Y, PLAN2X, PLAN2Y
	!	REAL(KIND=8)	 Lx, Ly, pi, 
	!	COMPLEX(kind=8)	omeg(1:Nx, lowyv(t_p):upyv(t_p)), omeghat(1:Ny, lowxv(t_p):upxv(t_p)),  &
	!					Kx(lowxv(t_p):upxv(t_p)), Ky(1:Ny), Kx2(lowxv(t_p):upxv(t_p)), Ky2(1:Ny), &
	!					T1(1:ydiff,1:Nx), T2(1:ydiff,1:Nx), ffttemp1, ffttemp2
	!
	! .. INPUT ..
	! Ny		= The number of grid points in the vertical direction 
	! Nx		= The number of grid points in the horizontal direction
	! info_p	= MPI error return integer
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! lowyv		= lower array index for data to be saved to file
	! upyv		= upper array index for data to be saved to file
	! lowxv		= lower array index for data to be saved to file
	! upxv		= upper array index for data to be saved to file
	! lowyv	= lower array index for data to be saved to file
	! upyv	= upper array index for data to be saved to file
	! lowxv	= lower array index for data to be saved to file
	! upxv	= upper array index for data to be saved to file
	! PLANX		= FFT plan in X direction
	! PLANY		= FFT plan in Y direction
	! Kx		= X fourier frequencies
	! Ky		= Y fourier frequencies
	! Kx2		= X fourier frequencies, modified to allow for division
	! Ky2		= Y fourier frequencies, modified to allow for division
	! Kx		= X fourier frequencies
	! kxt		= X fourier frequencies, to be written
	! Ky		= Y fourier frequencies
	! Ly		= Length in y direction
	! Lx		= Length in x direction
	! pi		= the number pi
	! omeg	= vorticity size: [Nx Ny/n_p]
	! omeghat = FFT(vorticity), (FFT in both spatial coordinates), size: [Ny Nx/n_p]
	! T1		= swap space for FFT
	! T2		= swap space for FFT
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!-----------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! fft2dFor:	-- Computes the 2d-FFT forward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once,
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	!-----------------------------------------------------------------------------------------------
	USE mpi

	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, info_p, xdiff, ydiff 
	INTEGER(KIND=8), INTENT(INOUT)					::  PLAN1X, PLAN1Y, PLAN2X, PLAN2Y
	REAL(KIND=8), INTENT(IN)						::  Lx, Ly, pi
	! .. Array Arguments ..
	INTEGER(KIND=4), DIMENSION(0:n_p-1), INTENT(IN)	::  lowxv, upxv, lowyv, upyv
	REAL(KIND=8), DIMENSION(t_p*xdiff+1:(t_p+1)*xdiff), INTENT(OUT)	::  kx,kx2
	REAL(KIND=8), DIMENSION(1:Ny), INTENT(OUT)						::  ky,ky2  
	COMPLEX(KIND=8), DIMENSION(1:Nx,t_p*ydiff+1:(t_p+1)*ydiff), INTENT(OUT)	::  omeg, ffttemp
	COMPLEX(KIND=8), DIMENSION(1:Ny,t_p*xdiff+1:(t_p+1)*xdiff), INTENT(OUT)	::  omeghat, ffttemphat  
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT)				::  T1, T2
	! .. Local Scalars ..
	INTEGER(KIND=4)													::  i, j, n, stat
	COMPLEX(KIND=8)													::  kxt
	REAL(KIND=8)													::  x,y,kxt2
	! .. Local Arrays ..
	CHARACTER*100													::  name_file, DataDirName
	! .. Executable statements ..
	CONTINUE

	CALL GET_ENVIRONMENT_VARIABLE(NAME="ddir",VALUE=DataDirName,STATUS=stat)
	IF (stat.ne.0) THEN
		IF(t_p.eq.0) THEN
			PRINT*,"Need to set environment variable ddir to the name of the directory for output"
		END IF
		STOP
	END IF
!	Fourier frequencies
	DO i = 1, Nx/2
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx(i) = 2.0d0*pi*REAL(i-1,KIND=8)
		ENDIF
	END DO
	IF ((Nx/2 + 1 .GE.lowxv(t_p)).AND.(Nx/2 + 1 .LE.upxv(t_p))) THEN
		Kx( Nx/2 + 1 ) = 0.0d0 
	ENDIF
	DO i = Nx/2+2, Nx  
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx( i) = -1.0d0*2.0d0*pi*REAL(1-i+Nx,KIND=8)  
		ENDIF
	END DO      
	DO j = 1, Ny/2
		Ky(j) = 2.0d0*pi*REAL(j-1,KIND(0d0)) 
	END DO
	Ky( Ny/2 + 1 ) = 0.0d0 
	DO j = 1, Ny/2-1
		Ky( Ny/2 + 1 + j ) = -Ky(1-j+Ny/2) 
	END DO    
	IF(t_p.eq.0) THEN
		! .. Save x locations .. 
		name_file = trim(DataDirName)//'/X.dat'
		OPEN(unit=11,FILE=name_file,status="UNKNOWN") 
		REWIND(11)
		DO i=1,NX
			WRITE(11,*) REAL(i-1,kind(0d0))/REAL(Nx,kind=8) 
		END DO
		CLOSE(11)
		
		! .. Save y locations ..
		name_file = trim(DataDirName)//'/Y.dat'
		OPEN(unit=12,FILE=name_file,status="UNKNOWN") 
		REWIND(12)
		DO j=1,NY
			WRITE(12,*) REAL(j-1,kind(0d0))/REAL(Ny,kind=8) 
		END DO
		CLOSE(12)

		! .. Save y frequencies ..
		name_file = trim(DataDirName)//'/KY.dat'
		OPEN(unit=13,FILE=name_file,status="UNKNOWN") 
		REWIND(13)
		DO j=1,NY
			WRITE(13,*) Ky(j)
		END DO
		CLOSE(13)	

		! .. Save x frequencies .. 
		name_file = trim(DataDirName)//'/KX.dat'
		OPEN(unit=14,FILE=name_file,status="UNKNOWN") 
		REWIND(14)
		DO i=1,NX
			IF((i.ge.1).and.(i.le.Nx/2)) THEN
				kxt2 = 2.0d0*pi*REAL(i-1,KIND=8)
			ELSEIF(i.eq.1+Nx/2) THEN
				kxt2 = 0.0d0
			ELSEIF((i.ge.Nx/2+1).and.(i.le.Nx)) THEN 
				kxt2 = -1.0d0*2.0d0*pi*REAL(1-i+Nx,KIND=8)
			END IF    			
			WRITE(14,*) kxt2
		END DO
		CLOSE(14)
	END IF

	IF ((1.GE.lowxv(t_p)).AND.(1.LE.upxv(t_p))) THEN
		Kx2(1) = 0.1d0
	ENDIF
	DO i = 2, Nx/2
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx2(i) = 2.0d0*pi*REAL(i-1,KIND=8)
		ENDIF
	END DO
	IF ((Nx/2 + 1 .GE.lowxv(t_p)).AND.(Nx/2 + 1 .LE.upxv(t_p))) THEN
		Kx2( Nx/2 + 1 ) = 0.10d0
	ENDIF
	DO i = Nx/2+2, Nx  
		IF ((i.GE.lowxv(t_p)).AND.(i.LE.upxv(t_p))) THEN
			Kx2( i) = -1.0d0*2.0d0*pi*REAL(1-i+Nx,KIND=8)  
		ENDIF
	END DO      
	DO j = 1, Ny/2
		Ky2(j) = 2.0d0*pi*REAL(j-1,KIND(0d0)) 
	END DO
	Ky2( Ny/2 + 1 ) = 0.10d0
	DO j = 1, Ny/2-1
		Ky2( Ny/2 + 1 + j ) = -Ky2(1-j+Ny/2) 
	END DO   
	Ky2(1)= 0.10d0
	!Initial data
	DO j = t_p*ydiff+1,(t_p+1)*ydiff
		y = REAL(j-1,kind(0d0))/REAL(Ny,kind=8) 
		DO i=1,Nx
			x = REAL(i-1,kind(0d0))/REAL(Nx,kind=8)
			omeg(i,j) = 4.0d0*pi*sin(2.0d0*pi*x)*sin(2.0d0*pi*y)
		END DO
	END DO   
		 
	CALL fft2dFor(Nx, Ny , n_p, t_p, xdiff, ydiff, info_p, &
			PLAN1X, PLAN1Y, &
			lowxv, upxv, lowyv, upyv, &
			omeg, omeghat, T1, T2, ffttemp, ffttemphat ) 


	END SUBROUTINE init
