	SUBROUTINE fft2dForEnd(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,PLAN1Y,VARS_d,T1_d,T2_d)     

	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT forward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dForEnd(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, PLANY, VARS_d, T1_d, T2_d  )  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p
	!	INTEGER(KIND=8)	PLANY
	!	COMPLEX(KIND=8)	 VARS_d(1:ydiff*Nx), T1_d((1:ydiff*Nx)), T2_d((1:ydiff*Nx))
	! 
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! PLANY		= Forward fft in Y direction
	! VARS_d 	= FFT(VARE), (FFT in both spatial coordinates), size: [Ny Nx/n_p]	
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! CUFFT library
	!---------------------------------------------------------------------------------------------
	USE mpi
	
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)									:: Nx, Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)								:: info_p
	INTEGER(KIND=8), INTENT(INOUT)								:: PLAN1Y
	! .. Array Arguments ..
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT) 		:: VARS_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT)			:: T1_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(IN)			:: T2_d
	! .. Local Scalars ..
	INTEGER(KIND=4)												:: i, j, n
	! .. Executable statements ..	
	CONTINUE
	
	DO n = 1,n_p
		DO j=1,ydiff
			DO i=1,xdiff
				T1_d(Ny*(i-1)+(n-1)*ydiff+j)=&
				T2_d((j-1)*xdiff+xdiff*ydiff*(n-1)+i)
			END DO
		END DO	
	END DO

	DO i = 1,xdiff
		CALL dfftw_execute_dft(PLAN1Y,T1_d((i-1)*Ny+1:i*Ny),VARS_d((i-1)*Ny+1:i*Ny))
	END DO


	END SUBROUTINE fft2dForEnd
