	SUBROUTINE fft2dForTranspose(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,T1_d,T2_d)     

	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT forward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dForTranspose(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamcopy, &
	!			 T1_d, T2_d ,T1, T2  )  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamcopy
	!	INTEGER(KIND=8)	PLANX, PLANY
	!	COMPLEX(KIND=8)	T1_d((1:ydiff*Nx)), T2_d((1:ydiff*Nx))
	! 
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	! T1		= swap space for FFT
	! T2		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	!---------------------------------------------------------------------------------------------
	USE mpi
  	
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)								:: Nx , Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)							:: info_p
	! .. Array Arguments ..
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(IN)		::  T1_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT)		::  T2_d
	! .. Local Scalars ..
	INTEGER(KIND=4)											::  istat,proc,nn
	! .. Local Arrays ..
	INTEGER(KIND=4), DIMENSION(1:2*n_p)						:: req1
	INTEGER(KIND=4), DIMENSION(MPI_STATUS_SIZE,2*n_p)		:: mpistatus
	! .. Executable statements ..	
	CONTINUE

	DO nn=1,n_p
		proc=mod(t_p+nn,n_p)	! processor to or receive from
		CALL MPI_RECV_INIT(T2_d(proc*xdiff*ydiff+1),xdiff*ydiff,&		
						  		MPI_DOUBLE_COMPLEX,proc,proc*n_p+t_p+n_p*n_p,&
						  		MPI_COMM_WORLD,req1(nn),info_p)
		CALL MPI_SEND_INIT(T1_d(proc*xdiff*ydiff+1),xdiff*ydiff,&
						 		MPI_DOUBLE_COMPLEX,proc,t_p*n_p+proc+n_p*n_p,&
								MPI_COMM_WORLD,req1(nn+n_p),info_p)
	END DO
	CALL MPI_STARTALL(2*n_p,req1,info_p)
	CALL MPI_WAITALL(2*n_p,req1,mpistatus,info_p)
	
	DO nn=1,2*n_p					
		CALL MPI_REQUEST_FREE(req1(nn),info_p)
	END DO
	
	END SUBROUTINE fft2dForTranspose
