	!
	! Define the INTERFACE to the NVIDIA CUFFT routines
	!
	! This follows examples taken from:
	!
	! http://cudamusing.blogspot.com/2010/05/calling-cufft-from-cuda-fortran.html

	module precision
	! Precision control

	integer(kind=4), parameter, public :: Single = kind(0.0) ! Single precision
	integer(kind=4), parameter, public :: Double = kind(0.0d0) ! Double precision

	integer(kind=4), parameter, public :: fp_kind = Double
	!integer(kind=4), parameter, public :: fp_kind = Single

	end module precision

	module cufft

	integer(kind=4), public :: CUFFT_FORWARD = -1
	integer(kind=4), public :: CUFFT_INVERSE = 1
	integer(kind=4), public :: CUFFT_R2C = Z'2a' ! Real to Complex (interleaved)
	integer(kind=4), public :: CUFFT_C2R = Z'2c' ! Complex (interleaved) to Real
	integer(kind=4), public :: CUFFT_C2C = Z'29' ! Complex to Complex, interleaved
	integer(kind=4), public :: CUFFT_D2Z = Z'6a' ! Double to Double-Complex
	integer(kind=4), public :: CUFFT_Z2D = Z'6c' ! Double-Complex to Double
	integer(kind=4), public :: CUFFT_Z2Z = Z'69' ! Double-Complex to Double-Complex

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftPlan1d(cufftHandle *plan, int nx,cufftType type,int batch)
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	interface cufftPlan1d
	subroutine cufftPlan1d(plan, nx, type, batch) bind(C,name='cufftPlan1d')
	use iso_c_binding
	integer(c_int):: plan
	integer(c_int),value:: nx, batch,type
	end subroutine cufftPlan1d
	end interface cufftPlan1d

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftPlanMany(cufftHandle *plan, int rank, int *n, int *inembed,
	! 	int istride, int idist, int *onembed, int ostride,
	! 	int odist, cufftType type, int batch);
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	interface cufftPlanMany
	subroutine cufftPlanMany(plan, rank, n, inembed, istride, idist, &
		onembed, ostride, odist, type, batch) bind(C,name='cufftPlanMany')
	use iso_c_binding
	integer(c_int):: plan
	integer(c_int),value:: rank, n, inembed, istride, idist
	integer(c_int),value:: onembed, ostride, odist, type, batch
	end subroutine cufftPlanMany
	end interface cufftPlanMany

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftDestroy(cufftHandle plan)
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	interface cufftDestroy
	subroutine cufftDestroy(plan) bind(C,name='cufftDestroy')
	use iso_c_binding
	integer(c_int),value:: plan
	end subroutine cufftDestroy
	end interface cufftDestroy

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftExecC2C(cufftHandle plan,
	! cufftComplex *idata,
	! cufftComplex *odata,
	! int direction)
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	interface cufftExecC2C
	subroutine cufftExecC2C(plan, idata, odata, direction) &
	& bind(C,name='cufftExecC2C')
	use iso_c_binding
	use precision
	integer(c_int),value:: direction
	integer(c_int),value:: plan
	complex(fp_kind),device:: idata(1:n*batch),odata(1:n*batch)
	end subroutine cufftExecC2C
	end interface cufftExecC2C

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftExecZ2Z(cufftHandle plan,
	! cufftDoubleComplex *idata,
	! cufftDoubleComplex *odata,
	! int direction);
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	interface cufftExecZ2Z
	subroutine cufftExecZ2Z(plan, idata, odata, direction) &
	& bind(C,name='cufftExecZ2Z')
	use iso_c_binding
	use precision
	integer(c_int),value:: direction
	integer(c_int),value:: plan
	complex(fp_kind),device:: idata(1:n*batch),odata(1:n*batch)
	end subroutine cufftExecZ2Z
	end interface cufftExecZ2Z

	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	!
	! cufftSetStream(cufftHandle plan,
	! int stream);
	!
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	interface cufftSetStream
	subroutine cufftSetStream(plan,stream) &
	& bind(C,name='cufftSetStream')
	use iso_c_binding
	use precision
	integer(c_int),value:: stream
	integer(c_int),value:: plan
	end subroutine cufftSetStream
	end interface cufftSetStream

	end module cufft
