	SUBROUTINE fftinit(	Nx, Ny, xdiff, ydiff, PLAN1X, PLAN1Y, PLAN2X, PLAN2Y )
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	fftinit initializes PLANX, PLANY,
	!	data structures containing all the information that 
	!	fftw needs in order to compute the 1D Fourier transform, 
	!	forward and backward, in x and y directions, respectively
	!
	! .. SPECIFICATION ..
	!
	!	fftinit( Nx, Ny, xdiff, ydiff, PLANX, PLANY )
	!
	!	INTEGER(kind=4)	Nx, Ny, xdiff, ydiff, PLANX, PLANY
	!	
	! .. INPUT ..
	! xdiff		= number of 1D arrays per processor in Y direction
	! ydiff		= number of 1D arrays per processor in X direction
	! Nx		= The number of grid points in the horizontal direction
	! Ny		= The number of grid points in the vertical direction 
	! PLANX		= Plan for fft in X direction
	! PLANY		= Plan for fft in Y direction
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! .. EXTERNAL ROUTINES REQUIRED ..
	!	MPI library
	!	FFTW3
	!---------------------------------------------------------------------------------------------

	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN) 		::  Nx, Ny, xdiff, ydiff 
	INTEGER(KIND=8), INTENT(OUT) 		::  PLAN1X, PLAN1Y, PLAN2X, PLAN2Y
	INTEGER(KIND=4), PARAMETER 		::  FFTW_BACKWARD=1, FFTW_FORWARD=-1, FFTW_ESTIMATE=64, FFTW_EXHAUSTIVE=8
	COMPLEX(KIND=8), DIMENSION(1:Nx) 		::  fftx1, fftx2
	COMPLEX(KIND=8), DIMENSION(1:Ny) 		::  ffty1, ffty2

	CONTINUE

   	CALL dfftw_plan_dft_1d(PLAN1X,Nx,fftx1(1:Nx),fftx2(1:Nx),FFTW_FORWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN2X,Nx,fftx2(1:Nx),fftx1(1:Nx),FFTW_BACKWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN1Y,Ny,ffty1(1:Ny),ffty2(1:Ny),FFTW_FORWARD,FFTW_ESTIMATE)
    CALL dfftw_plan_dft_1d(PLAN2Y,Ny,ffty2(1:Ny),ffty1(1:Ny),FFTW_BACKWARD,FFTW_ESTIMATE)
	
	END SUBROUTINE fftinit
