



		SUBROUTINE ENDWRITEBINFILE(LAVINI, info_p, n_p, t_p, lowyv, upyv, Nx, Ny, thefile)       
		!-----------------------------------------------------------------------------------------------------------------------------
		! PURPOSE
		!
		! ENDWRITEBINFILE finishes saving a two dimensional field in native binary, double format using asynchronous MPIIO 
		!
		! SPECIFICATION
		!
		! ENDWRITEBINFILE(LAVINI(1:NX,lowyv:upyv),info_p,n_p,t_p,lowyv(t_p),upyv(t_p),NX,NY,thefile)
		!
		! INTEGER		info_p, n_p, t_p,lowyv,upyv,NX,NY,thefile
		! DOUBLE PRECISION	LAVINI(1:NX,lowyv:upyv)
		!
		! .. INPUT ..
		! NY  		= The number of grid points in the vertical direction 
		! NX		= The number of grid points in the horizontal direction
		! info_p	= MPI error return integer
		! n_p		= Number of MPI processes
		! t_p		= Number of this MPI process
		! lowyv		= lower array index for data to be saved to file
		! upyv		= upper array index for data to be saved to file
		! thefile	= file handle 
		! LAVINI 	= Data to be saved to disk
		!
		! REFERENCES
		!
		! ACCURACY
		!		
		! ERROR INDICATORS AND WARNINGS
		!
		! FURTHER COMMENTS
		!-----------------------------------------------------------------------------------------------------------------------------------------
		! External routines required
		! MPI library
		!
		USE MPI
		! .. Implicit none statement ..		
		IMPLICIT NONE
		! .. Scalar arguments ..
		INTEGER(KIND=4), INTENT(IN)		 				:: n_p, t_p, lowyv, upyv, Nx, Ny 
		INTEGER(KIND=4), INTENT(INOUT)		 			:: info_p, thefile		
		! .. Array arguments ..
		DOUBLE PRECISION, INTENT(IN), DIMENSION(1:Nx,lowyv:upyv)  	:: LAVINI 
		! .. Local Scalars
		INTEGER, DIMENSION(MPI_STATUS_SIZE) 						:: mpistatus
		! .. Executable statements ..	
		CONTINUE		
      		CALL MPI_FILE_WRITE_ORDERED_END(thefile,LAVINI,mpistatus,info_p)
		CALL MPI_FILE_CLOSE(thefile, info_p)         
 
  		END SUBROUTINE ENDWRITEBINFILE
