	SUBROUTINE fftinit(	Nx, Ny, xdiff, ydiff, PLANX, PLANY )
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	fftinit initializes PLANX, PLANY,
	!	data structures containing all the information that 
	!	fftw needs in order to compute the 1D Fourier transform, 
	!	forward and backward, in x and y directions, respectively
	!
	! .. SPECIFICATION ..
	!
	!	fftinit( Nx, Ny, xdiff, ydiff, PLANX, PLANY )
	!
	!	INTEGER(kind=4)	Nx, Ny, xdiff, ydiff, PLANX, PLANY
	!	
	! .. INPUT ..
	! xdiff		= number of 1D arrays per processor in Y direction
	! ydiff		= number of 1D arrays per processor in X direction
	! Nx		= The number of grid points in the horizontal direction
	! Ny		= The number of grid points in the vertical direction 
	! PLANX		= Plan for fft in X direction
	! PLANY		= Plan for fft in Y direction
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! .. EXTERNAL ROUTINES REQUIRED ..
	!	MPI library
	!	FFTW3
	!---------------------------------------------------------------------------------------------
    USE precision
    USE cufft


	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN) 		::  Nx, Ny, xdiff, ydiff 
	INTEGER(KIND=4), INTENT(OUT) 		::  PLANX, PLANY
	INTEGER(KIND=4)						:: rank, n, inembed, istride, idist
	INTEGER(KIND=4)						:: onembed, ostride, odist, batch
	
	CONTINUE

  	! set up ffts
	rank=1
	n=Nx
	batch=ydiff
	inembed=Nx*ydiff
	idist=Nx
	istride=1
	onembed=Nx*ydiff
	ostride=1
	odist=Nx

	!CALL cufftPlanMany(PLANX,rank, n, inembed, istride, idist, &
	!					onembed, ostride, odist, CUFFT_Z2Z, batch)
	CALL cufftPlan1d(PLANX, n, CUFFT_Z2Z, batch)
	rank=1
	n=Ny
	batch=xdiff
	inembed=Ny*xdiff
	idist=Ny
	istride=1
	onembed=Ny*xdiff
	ostride=1
	odist=Ny
!	CALL cufftPlanMany(PLANY,rank, n, inembed, istride, idist, &
!						onembed, ostride, odist, CUFFT_Z2Z, batch)
	CALL cufftPlan1d(PLANY, n, CUFFT_Z2Z, batch)
	
	END SUBROUTINE fftinit
