
	SUBROUTINE parainit(	Nx, Ny, Re, Nt, plotgap,  &
				DT, time, t_p, n_p, info_p)

	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	parainit intialize parameters, which are stocked in the Input File 
	!
	! .. SPECIFICATION ..
	!
	!	parainit(	Nx, Ny, Re, Nt, plotgap, DT, eta, time, t_p, n_p, info_p)
	!	
	!	INTEGER(KIND=4)		Nx, Ny, Nt, plotgap, t_p, n_p, info_p 
	!	DOUBLE PRECISION	Re, DT, time
	!	
	! .. INPUT ..
	!	Nx				= power of two, number of modes in the x direction
	!	Ny				= power of two, number of modes in the y direction
	!	Re				= Reynolds number
	!	Nt				= the number of timesteps
	!	plotgap			= the number of timesteps to take before plotting
	!	t_p				= number of MPI process
	!	n_p				= total number of MPI processes
	!	info_p			= MPI error output variable
	!	DT				= the time step
	!	time			= initial time of computation
	!	InputFileName	= name of the Input File
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! 
	!---------------------------------------------------------------------------------------------
	USE mpi
	IMPLICIT NONE
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)			::  t_p, n_p	
	INTEGER(KIND=4), INTENT(OUT)		::  Nx, Ny, Nt, plotgap, info_p 
	DOUBLE PRECISION, INTENT(OUT)		::  Re, DT, time
	! .. Local scalars ..
	INTEGER(KIND=4)						::  stat
	! .. Local Arrays ..
	CHARACTER*40						::  InputFileName
	INTEGER(KIND=4), DIMENSION(1:5)		::  intcomm
	DOUBLE PRECISION, DIMENSION(1:3)	::  dpcomm

	IF(t_p.eq.0) THEN
		CALL GET_ENVIRONMENT_VARIABLE(NAME="inputfile",VALUE=InputFileName, STATUS=stat)
		IF(stat.NE.0) THEN
			PRINT*,"Need to set environment variable inputfile to the name of the file where the simulation parameters are set"
			STOP
		END IF
		OPEN(unit=11,FILE=trim(InputFileName),status="OLD") 
		REWIND(11)
		READ(11,*) intcomm(1), intcomm(2), dpcomm(1),  intcomm(3), intcomm(4),&
			 	dpcomm(2), dpcomm(3)
		CLOSE(11)
		PRINT *,"Read inputfile"
	END IF
	CALL MPI_BCAST(intcomm,5,MPI_INTEGER,0,MPI_COMM_WORLD,info_p)
	CALL MPI_BCAST(dpcomm,3,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,info_p)
	Nx=intcomm(1)
	Ny=intcomm(2)
	Nt=intcomm(3)
	plotgap=intcomm(4)
	Re=dpcomm(1)
	DT=dpcomm(2)
	time=dpcomm(3)
	
	IF(t_p.eq.0) THEN
		PRINT *,'Nx ',Nx
		PRINT *,'Ny ',Ny
		PRINT *,'Nt ',Nt
		PRINT *,'plotgap ',plotgap
		PRINT *,'Re ',Re
		PRINT *,'DT ',DT
		PRINT *,'time ',time			
	END IF
	
	END SUBROUTINE parainit
