	SUBROUTINE fft2dForStart(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,streamCompute,PLANX,VARE_d,T1_d,T2_d)     

	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT forward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dForStart(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamCompute, &
	!			PLANX, VARE_d, T1_d, T2_d  )  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamCompute
	!	INTEGER(KIND=8)	PLANX
	!	COMPLEX(KIND=8)	VARE_d(1:ydiff*Nx),T1_d((1:ydiff*Nx)),T2_d((1:ydiff*Nx))
	! 
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! PLANX		= Forward fft in X direction
	! VARE_d	= ARRAY to be transformed, size: [Nx Ny/n_p]
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! FFTW library
	!---------------------------------------------------------------------------------------------
	USE mpi
    USE precision
    USE cufft
    USE cudafor
	
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						:: Nx , Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					:: info_p,streamCompute
	INTEGER(KIND=4), INTENT(INOUT)					:: PLANX
	! .. Array Arguments ..
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN)  	::  VARE_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT)		::  T1_d, T2_d
	! .. Local Scalars ..
	INTEGER(KIND=4)								::  i, j, n
	! .. Executable statements ..	
	CONTINUE

	CALL cufftSetStream(PLANX,streamCompute)
	CALL cufftExecZ2Z(PLANX,VARE_d,T2_d,CUFFT_FORWARD)
	
	!$cuf kernel do(2) <<< *, *,stream=streamCompute >>>
	DO n = 1,n_p
		DO i=1,xdiff
			DO j=1,ydiff
				T1_d((j-1)*xdiff+(n-1)*ydiff*xdiff+i)=&
				T2_d(Nx*(j-1)+(n-1)*xdiff+i)
			END DO
		END DO
	END DO

	
	END SUBROUTINE fft2dForStart
