	SUBROUTINE fft2dForTranspose(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,streamcopy,T1_d,T2_d,T1,T2)     

	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT forward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dForTranspose(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamcopy, &
	!			 T1_d, T2_d ,T1, T2  )  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamcopy
	!	INTEGER(KIND=8)	PLANX, PLANY
	!	COMPLEX(KIND=8)	T1_d((1:ydiff*Nx)), T2_d((1:ydiff*Nx)), T1((1:ydiff*Nx)), T2((1:ydiff*Nx))
	! 
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	! T1		= swap space for FFT
	! T2		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	!---------------------------------------------------------------------------------------------
	USE mpi
    USE precision
 	USE cudafor
 	
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						:: Nx , Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					:: info_p,streamcopy
	! .. Array Arguments ..
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN)	::  T1_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT)	::  T2_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(OUT), PINNED	::  T1, T2
	! .. Local Scalars ..
	INTEGER(KIND=4)													::  istat
	! .. Executable statements ..	
	CONTINUE

	! transfer data to CPU then do transposition
	!T1=T1_d
	istat = cudaMemcpyAsync(T1,T1_d,ydiff*Nx,streamcopy)
	istat = cudaStreamSynchronize(streamcopy)
	
	CALL MPI_ALLTOALL(T1(1), ydiff*xdiff, MPI_DOUBLE_COMPLEX, T2(1), ydiff*xdiff,&
				MPI_DOUBLE_COMPLEX, MPI_COMM_WORLD, info_p)
	! transfer data from CPU
	!T2_d=T2
	istat = cudaMemcpyAsync(T2_d,T2,ydiff*Nx,streamcopy)
	
	END SUBROUTINE fft2dForTranspose
