	SUBROUTINE	compandsave( 	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
					PLANX, PLANY, plotnum, thefileu, thefilev, TIME,&
					 lowxv, upxv, lowyv, upyv, &
					omeg_d, omeghat_d, &
					PLOTARRAYU_d, PLOTARRAYV_d, PLOTARRAYU,  PLOTARRAYV, T1_d, T2_d, T1, T2)
					  
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	compandsave saves the Fourier coefficients and the numerical solution
	!	in a binary file, by calling the subroutine writebinfile
	!
	! .. SPECIFICATION ..
	!	compandsave( Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, &
	!			PLANX, PLANY, plotnum, thefileu, thefilev, TIME,&
	!			lowxv, upxv, lowyv, upyv, &
	!			omeg_d, omeghat_d, PLOTARRAYU_d, PLOTARRAYV_d, PLOTARRAYU, &
	!			PLOTARRAYV, T1_d, T2_d, T1, T2  ) 
	!
	!	INTEGER(kind=4)	lowxv(0:n_p-1), upxv(0:n_p-1), lowyv(0:n_p-1), upyv(0:n_p-1), &
	!			Nx, Ny, n_p, t_p, xdiff, ydiff, info_p, thefileu, thefilev
	!	INTEGER(kind=8)	PLAN2X, PLAN2Y, PLAN1X, PLAN1Y
	!	REAL(KIND=8)	PLOTARRAYU(1:Nx,lowyv(t_p):upyv(t_p)), &
	!				PLOTARRAYV(1:NY,lowxv(t_p):upxv(t_p))
	!	COMPLEX(kind=8)	omeg_d(1:Nx, lowyv(t_p):upyv(t_p)), omeghat_d(1:Nx, lowyv(t_p):upyv(t_p)),
	!			T1_d(1:ydiff,1:Nx), T2_d(1:ydiff,1:Nx), T1(1:ydiff,1:Nx), T2(1:ydiff,1:Nx)
	!
	! .. INPUT ..
	! Ny  		= The number of grid points in the vertical direction 
	! Nx		= The number of grid points in the horizontal direction
	! info_p	= MPI error return integer
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! lowyv		= lower array index for data to be saved to file
	! upyv		= upper array index for data to be saved to file
	! lowxv		= lower array index for data to be saved to file
	! upxv		= upper array index for data to be saved to file
	! PLANX		= Forward fft in X direction
	! PLANY		= Forward fft in Y direction
	! Kx		= X fourier frequencies
	! Ky		= Y fourier frequencies
	! TIME		= Time 
	! omeg_d	= vorticity size: [Nx Ny/n_p]
	! omeghat_d = FFT(vorticity), (FFT in both spatial coordinates), size: [Ny Nx/n_p]
	! PLOTARRAYU	= Array used to save real data for plotting; size: [Nx Ny/n_p]
	! PLOTARRAYV	= Array used to save real data for plotting; size: [Ny Nx/n_p]
	!
	! REFERENCES
	!
	! ACCURACY
	!		
	! ERROR INDICATORS AND WARNINGS
	!
	! FURTHER COMMENTS
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! writebinfile	-- saves a two dimensional field in native binary, double format using MPIIO
	! fft2dFor  	-- Computes the 2d-FFT forward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	! fft2dBack	-- Computes the 2d-FFT backward in both spatial coordinates, by using 
	!		MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once
	!		so output is in the opposite orientation to input.  Input data is destroyed.
	!---------------------------------------------------------------------------------------------

	USE mpi
	IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					::  info_p, thefileu, thefilev, plotnum
	INTEGER(KIND=4), INTENT(INOUT)					::  PLANX, PLANY
	REAL(KIND=8), INTENT(IN)						::  time  
	! .. Array Arguments ..
	INTEGER(KIND=4), DIMENSION(0:n_p-1), INTENT(IN)	::  lowxv, upxv, lowyv, upyv
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN) 		::  omeg_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN) 		::  omeghat_d
	REAL(KIND=8), DEVICE, DIMENSION(1:Nx,lowyv(t_p):upyv(t_p)), INTENT(OUT) 		::  PLOTARRAYU_d
	REAL(KIND=8), DEVICE, DIMENSION(1:Ny,lowxv(t_p):upxv(t_p)), INTENT(OUT) 		::  PLOTARRAYV_d
	REAL(KIND=8), DIMENSION(1:Nx,lowyv(t_p):upyv(t_p)), INTENT(OUT) 				::  PLOTARRAYU
	REAL(KIND=8), DIMENSION(1:Ny,lowxv(t_p):upxv(t_p)), INTENT(OUT) 				::  PLOTARRAYV
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(INOUT)					::  T1_d, T2_d
	COMPLEX(KIND=8), DIMENSION(1:ydiff*Nx), INTENT(INOUT)							::  T1, T2
	! .. Local scalars ..
	INTEGER(KIND=4)								:: i, j, n, ind, stat
	! .. Local arrays ..
	CHARACTER*100								::  nomfile, DataDirName
	CHARACTER*10								::  number_file
	! .. Executable statements ..	
	CONTINUE

	!$cuf kernel do(2) <<< *, * >>>
	DO j=t_p*ydiff+1,(t_p+1)*ydiff 
		DO i=1,Nx
			PLOTARRAYU_d(i,j) = REAL(OMEG_d(Nx*(j-(t_p*ydiff+1))+i),kind=8)
		END DO
	END DO
	! Transfer data to host
	PLOTARRAYU=PLOTARRAYU_d
!	Save data
!	"Recuperation" of the name of the directory where data have to be saved
	CALL GET_ENVIRONMENT_VARIABLE(NAME="ddir", VALUE=DataDirName, STATUS=stat)
	IF (stat.NE.0) STOP
	nomfile = trim(DataDirName)//'/u'
	ind = index(nomfile,' ') - 1
	WRITE(number_file,'(i0)') 10000000+plotnum
	nomfile = nomfile(1:ind)//number_file
	ind = index(nomfile,' ') - 1
	nomfile = nomfile(1:ind)//'.datbin'
	CALL WRITEBINFILE(NX,NY,lowyv(t_p),upyv(t_p),n_p,t_p,nomfile,thefileu,info_p,PLOTARRAYU) 

!	Save datas
	nomfile = trim(DataDirName)//'/v'
	ind = index(nomfile,' ') - 1
	WRITE(number_file,'(i0)') 10000000+plotnum
	nomfile = nomfile(1:ind)//number_file
	ind = index(nomfile,' ') - 1
	nomfile = nomfile(1:ind)//'.datbin'	
	!$cuf kernel do(2) <<< *, * >>>
	DO i = t_p*xdiff+1,(t_p+1)*xdiff
		DO j=1,Ny
			PLOTARRAYV_d(j,i)=LOG10(ABS(OMEGHAT_d(Ny*(i-(t_p*xdiff+1))+j))) 
		END DO
	END DO
		! Transfer data to host
	PLOTARRAYV=PLOTARRAYV_d

	CALL WRITEBINFILE(NY,NX,lowxv(t_p),upxv(t_p),n_p,t_p,nomfile,thefilev,info_p,PLOTARRAYV) 
			 

	IF (t_p.EQ.0) THEN
		! .. Save Time and Mass ..
		nomfile = trim(DataDirName)//'/Time.dat'
		IF (plotnum.eq.0) THEN
			! .. If this is the first plot, start at the beginning of the file ..
			OPEN(unit=1,FILE=nomfile,status="UNKNOWN") 
			REWIND(1)
			WRITE(1,'(e15.3)') time
			CLOSE(1)
		ELSE	
			OPEN(unit=1,FILE=nomfile,status="OLD",position="APPEND") 
			WRITE(1,'(e15.3)') time
			CLOSE(1)
		END IF		
		
	ENDIF
	plotnum=plotnum+1	

	END SUBROUTINE compandsave

