	SUBROUTINE fft2dBackEnd(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,streamCompute,PLANX,VARS_d,T1_d,T2_d)  
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT backward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dBackEnd(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,PLANX,VARS,T1_d,T2_d)  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamCompute
	!	INTEGER(KIND=8)	PLANX
	!	COMPLEX(KIND=8)	 VARS(1:ydiff*Nx),T1_d(1:ydiff,1:Nx),T2_d(1:ydiff,1:Nx)
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! PLANX		= fft plan in X direction
	! VARS_d 	= FFT(VARE), (FFT in both spatial coordinates), size: [Ny Nx/n_p]	
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! CUFFT library
	!---------------------------------------------------------------------------------------------
	USE mpi
    USE precision
    USE cufft
	USE cudafor
	
	!IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					::  info_p,streamCompute
	INTEGER(KIND=4), INTENT(INOUT)					::  PLANX
	! .. Array Arguments ..
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT) ::  VARS_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN)				::  T1_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT)				::  T2_d
	! .. Local Scalars ..
	INTEGER(KIND=4)															::  i, j, n
	! .. Executable statements ..	
	CONTINUE

	!$cuf kernel do(2) <<< *, *,stream=streamCompute >>>
	DO n = 1,n_p
		DO i=1,xdiff
			DO j=1,ydiff
				T2_d(Nx*(j-1)+(n-1)*xdiff+i)=&
				T1_d((j-1)*xdiff+ydiff*xdiff*(n-1)+i)
			END DO
		END DO	
	END DO

	CALL cufftSetStream(PLANX,streamCompute)
	CALL cufftExecZ2Z(PLANX,T2_d,VARS_d,CUFFT_INVERSE)

	END SUBROUTINE fft2dBackEnd
