SUBROUTINE fft2dBackStart(Nx,Ny,n_p,t_p,xdiff,ydiff,info_p,streamCompute,PLANY,VARE_d,T1_d,T2_d)  
	!---------------------------------------------------------------------------------------------
	! .. PURPOSE ..
	!	Computes the 2d-FFT backward in both spatial coordinates, by using 
	!	MPI_ALL_TO_ALL to perform the data transposition. Data is only transposed once, so
	!	output is in the opposite orientation to input. Input data is destroyed.
	!
	! .. SPECIFICATION ..
	!	fft2dBack(	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p,streamCompute, &
	!			PLANX, PLANY, VARE, VARS, T1_d, T2_d  )  
	!	INTEGER(KIND=4)	Nx, Ny, n_p, t_p, xdiff, ydiff, info_p
	!	INTEGER(KIND=8)	PLANY
	!	COMPLEX(KIND=8)	VARE(1:ydiff*Nx),T1_d(1:ydiff,1:Nx),T2_d(1:ydiff,1:Nx)
	! .. INPUT ..
	! Nx		= The number of grid points in the horizontal direction
	! Ny  		= The number of grid points in the vertical direction 
	! n_p		= Number of MPI processes
	! t_p		= Number of this MPI process
	! info_p	= MPI error return integer
	! PLANY		= fft plan in Y direction
	! VARE_d	= ARRAY to be transformed, size: [Nx Ny/n_p]
	! T1_d		= swap space for FFT
	! T2_d		= swap space for FFT
	!---------------------------------------------------------------------------------------------
	! EXTERNAL ROUTINES REQUIRED
	! MPI library
	! cufft library
	!---------------------------------------------------------------------------------------------
	USE mpi
    USE precision
    USE cufft
	USE cudafor
	
	!IMPLICIT NONE    
	! .. Scalar Arguments ..
	INTEGER(KIND=4), INTENT(IN)						::  Nx, Ny, n_p, t_p, xdiff, ydiff
	INTEGER(KIND=4), INTENT(INOUT)					::  info_p,streamCompute
	INTEGER(KIND=4), INTENT(INOUT)					::  PLANY
	! .. Array Arguments ..
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(IN) ::  VARE_d
	COMPLEX(KIND=8), DEVICE, DIMENSION(1:ydiff*Nx), INTENT(OUT)				::  T1_d, T2_d
	! .. Local Scalars ..
	INTEGER(KIND=4)															::  i, j, n
	! .. Executable statements ..	
	CONTINUE

	CALL cufftSetStream(PLANY,streamCompute)
	CALL cufftExecZ2Z(PLANY,VARE_d,T1_d,CUFFT_INVERSE)
		
	!$cuf kernel do(2) <<< *, *,stream=streamCompute >>>
	DO n = 1,n_p
		DO j=1,ydiff
			DO i=1,xdiff
				T2_d((j-1)*xdiff+(n-1)*xdiff*ydiff+i)=&
				T1_d(Ny*(i-1)+(n-1)*ydiff+j)/REAL(Nx*Ny,kind=8)
			END DO
		END DO
	END DO
		
	END SUBROUTINE fft2dBackStart
