!--------------------------------------------------------------------
!
!
! PURPOSE
!
! This subroutine copies arrays
! current->old, new->current
!
! AUTHORS
!
! B. Cloutier, B.K. Muite, P. Rigge
! 4 June 2012
!
! INPUT
!
! .. Parameters ..
!  Nx                           = number of modes in x - power of 2 for FFT
!  Ny                           = number of modes in y - power of 2 for FFT
!  .. Arrays ..
!  unew                         = approximate solution
!  u                            = approximate solution
!  uold                         = approximate solution
!  vnew                         = Fourier transform of approximate solution
!  v                            = Fourier transform of approximate solution
!  vold                         = Fourier transform of approximate solution
!
! OUTPUT
!
!  u                            = approximate solution
!  v                            = Fourier transform of approximate solution
!  uold                         = approximate solution
!  vold                         = Fourier transform of approximate solution
!
! LOCAL VARIABLES
!
! .. Scalars ..
!  i                            = loop counter in x direction
!  j                            = loop counter in y direction
!
! REFERENCES
!
! ACKNOWLEDGEMENTS
!
! ACCURACY
!
! ERROR INDICATORS AND WARNINGS
!
! FURTHER COMMENTS
!--------------------------------------------------------------------
! External routines required
!
! External libraries required
! OpenMP library
SUBROUTINE storeold(Nx,Ny,unew,u,uold,vnew,v,vold)

  USE omp_lib
  ! Declare variables
  IMPLICIT NONE
  INTEGER(KIND=4), INTENT(IN)                             :: Nx,Ny
  REAL   (KIND=8), DIMENSION(1:NX,1:NY), INTENT(OUT)      :: uold
  COMPLEX(KIND=8), DIMENSION(1:NX/2+1,1:NY), INTENT(OUT)  :: vold
  REAL   (KIND=8), DIMENSION(1:NX,1:NY), INTENT(INOUT)    :: u
  REAL   (KIND=8), DIMENSION(1:NX,1:NY), INTENT(IN)       :: unew
  COMPLEX(KIND=8), DIMENSION(1:NX/2+1,1:NY), INTENT(INOUT):: v
  COMPLEX(KIND=8), DIMENSION(1:NX/2+1,1:NY), INTENT(IN)   :: vnew
  INTEGER(kind=4)                                         :: i,j

  !$OMP PARALLEL PRIVATE(i,j)

  !$OMP DO SCHEDULE(static)
  DO j=1,Ny
     DO i=1,Nx/2+1
        vold(i,j)=v(i,j)
     END DO
  END DO
  !$OMP END DO NOWAIT

  !$OMP DO SCHEDULE(static)
  DO j=1,Ny
     DO i=1,Nx
        uold(i,j)=u(i,j)
     END DO
  END DO
  !$OMP END DO NOWAIT

  !$OMP DO SCHEDULE(static)
  DO j=1,Ny
     DO i=1,Nx
        u(i,j)=unew(i,j)
     END DO
  END DO
  !$OMP END DO NOWAIT

  !$OMP DO SCHEDULE(static)
  DO j=1,Ny
     DO i=1,Nx/2+1
        v(i,j)=vnew(i,j)
     END DO
  END DO
  !$OMP END DO NOWAIT

  !$OMP END PARALLEL

END SUBROUTINE storeold
