% A program to solve the 2D Klein Gordon equation using a
% second order implicit method 

clear all; format compact; format short;
set(0,'defaultaxesfontsize',30,'defaultaxeslinewidth',.7,...
    'defaultlinelinewidth',6,'defaultpatchlinewidth',3.7,...
    'defaultaxesfontweight','bold')

% set up grid
tic
Lx = 5;         % period  2*pi*L
Ly = 5;         % period  2*pi*L
Nx = 2*256;     % number of harmonics
Ny = 2*256;     % number of harmonics
Nt = 500;       % number of time slices
dt = 0.001;     % time step
plotgap=10;     % timesteps between plots


% initialise variables
x = (2*pi/Nx)*(-Nx/2:Nx/2 -1)'*Lx;          % x coordinate
kx = 1i*[0:Nx/2-1 0 -Nx/2+1:-1]'/Lx;        % wave vector
y = (2*pi/Ny)*(-Ny/2:Ny/2 -1)'*Ly;          % y coordinate
ky = 1i*[0:Ny/2-1 0 -Ny/2+1:-1]'/Ly;        % wave vector
[xx,yy]=meshgrid(x,y);
[kxm,kym]=meshgrid(kx,ky);

% initial conditions
u = (0.5*exp(-1.0*(xx.^2+yy.^2))); %.*sin(10*xx+12*yy);
uold=u;
v=fft2(u); 
vold=fft2(uold); 
figure(1); clf; mesh(xx,yy,u); drawnow;
t=0; tdata(1)=t;

% initial energy
vx=0.5*kxm.*(v+vold);
vy=0.5*kym.*(v+vold);
ux=ifft2(vx); 
uy=ifft2(vy); 
ux=ifft2(vx); 
uy=ifft2(vy); 
Kineticenergy=0.5*abs( (u-uold)/dt).^2;           
Strainenergy=0.5*abs(ux).^2 +0.5*abs(uy).^2;           
Potentialenergy=1-cos(0.5*(u+uold));           
Kineticenergy=fft2(Kineticenergy);
Potentialenergy=fft2(Potentialenergy);
Strainenergy=fft2(Strainenergy);
EnKin(1)=Kineticenergy(1,1);
EnPot(1)=Potentialenergy(1,1);
EnStr(1)=Strainenergy(1,1);
En(1)=EnStr(1)+EnKin(1)+EnPot(1);
En0=En(1)
plotnum=1;

% solve pde and plot results

for n =1:Nt+1
    nonlin=sin(u);
    nonlinhat=fft2(nonlin);
    vnew=(0.25*(kxm.^2 + kym.^2 ).*(2*v+vold)...
             + (2*v-vold)/(dt*dt) - nonlinhat)./...
            (1/(dt*dt) - (kxm.^2 +  kym.^2)*0.25 );
    unew=ifft2(vnew);        
    t=n*dt;
    if (mod(n,plotgap)==0)
        figure(1); clf; mesh(xx,yy,real(u));
        t
        tdata(plotnum+1)=t;
        vx=0.5*kxm.*(v+vold);
        vy=0.5*kym.*(v+vold);
        ux=ifft2(vx); 
        uy=ifft2(vy); 
        Kineticenergy=0.5*abs( (unew-u)/dt).^2;           
        Strainenergy=0.5*abs(ux).^2 +0.5*abs(uy).^2;           
        Potentialenergy=1-cos(0.5*(u+uold));           
        Kineticenergy=fft2(Kineticenergy);
        Potentialenergy=fft2(Potentialenergy);
        Strainenergy=fft2(Strainenergy);
        EnKin(1+plotnum)=Kineticenergy(1,1);
        EnPot(1+plotnum)=Potentialenergy(1,1);
        EnStr(1+plotnum)=Strainenergy(1,1);
        En(1+plotnum)=EnStr(1+plotnum)+EnKin(1+plotnum)+EnPot(1+plotnum);
 
        Enchange(plotnum)=log(abs(1-En(1+plotnum)/En0));
        plotnum=plotnum+1;
    end
    % update old terms
    vold=v;
    v=vnew;
    uold=u;
    u=unew;
end
figure(5); clf; plot(tdata,En,'r-',tdata,EnKin,'b:',tdata,EnPot,'g-.',tdata,EnStr,'y--'); 
xlabel time; ylabel Energy; legend('Total','Kinetic','Potential','Strain');
figure(6); clf; plot(tdata(2:end),Enchange,'r-'); xlabel time; ylabel('Energy change'); 

figure(4); clf; mesh(xx,yy,abs(u).^2);
toc